@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 2.1.0 (_PROJECT_BUILD_ID_)
+####  Version 2.2.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -71,7 +71,7 @@ find.interaction.rfsrc <- function(
   nvar = NULL, 
   nrep  = 1,
   subset,
-  na.action = c("na.omit", "na.impute", "na.random"),
+  na.action = c("na.omit", "na.impute"),
   seed = NULL,
   do.trace = FALSE,
   verbose = TRUE,
