@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 2.1.0 (_PROJECT_BUILD_ID_)
+####  Version 2.2.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -65,7 +65,7 @@ predict.rfsrc <-
            newdata,
            outcome.target=NULL,
            importance = c(FALSE, TRUE, "none", "permute", "random", "anti", "permute.ensemble", "random.ensemble", "anti.ensemble")[4],
-           na.action = c("na.omit", "na.impute", "na.random"),
+           na.action = c("na.omit", "na.impute"),
            outcome = c("train", "test"),
            proximity = FALSE,
            var.used = c(FALSE, "all.trees", "by.tree"),
