@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 2.1.0 (_PROJECT_BUILD_ID_)
+####  Version 2.2.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -77,7 +77,7 @@ var.select.rfsrc <-
            xvar.wt = NULL,
            refit = (method != "md"),
            fast = FALSE,
-           na.action = c("na.omit", "na.impute", "na.random"),
+           na.action = c("na.omit", "na.impute"),
            always.use = NULL,  
            nrep = 50,        
            K = 5,             
